@@ -1,6 +1,7 @@
 /* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -24,6 +25,8 @@
 #include <thunar/thunar-file-monitor.h>
 #include <thunar/thunar-folder.h>
 #include <thunar/thunar-gobject-extensions.h>
+#include <thunar/thunar-io-jobs.h>
+#include <thunar/thunar-job.h>
 #include <thunar/thunar-private.h>
 
 
@@ -32,6 +35,7 @@
 enum
 {
   PROP_0,
+  PROP_CORRESPONDING_FILE,
   PROP_LOADING,
 };
 
@@ -46,20 +50,22 @@ enum
 
 
 
-static void     thunar_folder_class_init                  (ThunarFolderClass      *klass);
-static void     thunar_folder_init                        (ThunarFolder           *folder);
 static void     thunar_folder_finalize                    (GObject                *object);
 static void     thunar_folder_get_property                (GObject                *object,
                                                            guint                   prop_id,
                                                            GValue                 *value,
                                                            GParamSpec             *pspec);
-static void     thunar_folder_error                       (ThunarVfsJob           *job,
+static void     thunar_folder_set_property                (GObject                *object,
+                                                           guint                   prop_uid,
+                                                           const GValue           *value,
+                                                           GParamSpec             *pspec);
+static void     thunar_folder_error                       (ExoJob                 *job,
                                                            GError                 *error,
                                                            ThunarFolder           *folder);
-static gboolean thunar_folder_infos_ready                 (ThunarVfsJob           *job,
-                                                           GList                  *infos,
+static gboolean thunar_folder_files_ready                 (ThunarJob              *job,
+                                                           GList                  *files,
                                                            ThunarFolder           *folder);
-static void     thunar_folder_finished                    (ThunarVfsJob           *job,
+static void     thunar_folder_finished                    (ExoJob                 *job,
                                                            ThunarFolder           *folder);
 static void     thunar_folder_file_changed                (ThunarFileMonitor      *file_monitor,
                                                            ThunarFile             *file,
@@ -67,11 +73,10 @@ static void     thunar_folder_file_changed                (ThunarFileMonitor
 static void     thunar_folder_file_destroyed              (ThunarFileMonitor      *file_monitor,
                                                            ThunarFile             *file,
                                                            ThunarFolder           *folder);
-static void     thunar_folder_monitor                     (ThunarVfsMonitor       *monitor,
-                                                           ThunarVfsMonitorHandle *handle,
-                                                           ThunarVfsMonitorEvent   event,
-                                                           ThunarVfsPath          *handle_path,
-                                                           ThunarVfsPath          *event_path,
+static void     thunar_folder_monitor                     (GFileMonitor           *monitor,
+                                                           GFile                  *file,
+                                                           GFile                  *other_file,
+                                                           GFileMonitorEvent       event_type,
                                                            gpointer                user_data);
 
 
@@ -93,52 +98,25 @@ struct _ThunarFolder
 {
   GtkObject __parent__;
 
-  ThunarVfsJob           *job;
+  ThunarJob         *job;
 
-  ThunarFile             *corresponding_file;
-  GList                  *new_files;
-  GList                  *files;
+  ThunarFile        *corresponding_file;
+  GList             *new_files;
+  GList             *files;
 
-  ThunarFileMonitor      *file_monitor;
+  ThunarFileMonitor *file_monitor;
 
-  ThunarVfsMonitor       *monitor;
-  ThunarVfsMonitorHandle *handle;
+  GFileMonitor      *monitor;
 };
 
 
 
-static GObjectClass *thunar_folder_parent_class;
-static guint         folder_signals[LAST_SIGNAL];
-static GQuark        thunar_folder_quark;
+static guint  folder_signals[LAST_SIGNAL];
+static GQuark thunar_folder_quark;
 
 
 
-GType
-thunar_folder_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarFolderClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_folder_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarFolder),
-        0,
-        (GInstanceInitFunc) thunar_folder_init,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_OBJECT, I_("ThunarFolder"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarFolder, thunar_folder, GTK_TYPE_OBJECT)
 
 
 
@@ -147,12 +125,25 @@ thunar_folder_class_init (ThunarFolderClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_folder_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_folder_finalize;
   gobject_class->get_property = thunar_folder_get_property;
+  gobject_class->set_property = thunar_folder_set_property;
+
+  /**
+   * ThunarFolder::corresponding-file:
+   *
+   * The #ThunarFile referring to the #ThunarFolder.
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_CORRESPONDING_FILE,
+                                   g_param_spec_object ("corresponding-file",
+                                                        "corresponding-file",
+                                                        "corresponding-file",
+                                                        THUNAR_TYPE_FILE,
+                                                        G_PARAM_READABLE 
+                                                        | G_PARAM_WRITABLE
+                                                        | G_PARAM_CONSTRUCT_ONLY));
 
   /**
    * ThunarFolder::loading:
@@ -228,8 +219,7 @@ thunar_folder_init (ThunarFolder *folder)
   g_signal_connect (G_OBJECT (folder->file_monitor), "file-changed", G_CALLBACK (thunar_folder_file_changed), folder);
   g_signal_connect (G_OBJECT (folder->file_monitor), "file-destroyed", G_CALLBACK (thunar_folder_file_destroyed), folder);
 
-  /* connect to the file alteration monitor */
-  folder->monitor = thunar_vfs_monitor_get_default ();
+  folder->monitor = NULL;
 }
 
 
@@ -240,20 +230,22 @@ thunar_folder_finalize (GObject *object)
   ThunarFolder *folder = THUNAR_FOLDER (object);
 
   /* disconnect from the ThunarFileMonitor instance */
-  g_signal_handlers_disconnect_matched (G_OBJECT (folder->file_monitor), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
-  g_object_unref (G_OBJECT (folder->file_monitor));
+  g_signal_handlers_disconnect_matched (folder->file_monitor, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
+  g_object_unref (folder->file_monitor);
 
   /* disconnect from the file alteration monitor */
-  if (G_LIKELY (folder->handle != NULL))
-    thunar_vfs_monitor_remove (folder->monitor, folder->handle);
-  g_object_unref (G_OBJECT (folder->monitor));
+  if (G_LIKELY (folder->monitor != NULL))
+    {
+      g_file_monitor_cancel (folder->monitor);
+      g_object_unref (folder->monitor);
+    }
 
   /* cancel the pending job (if any) */
   if (G_UNLIKELY (folder->job != NULL))
     {
       g_signal_handlers_disconnect_matched (folder->job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
-      thunar_vfs_job_cancel (folder->job);
-      g_object_unref (G_OBJECT (folder->job));
+      g_object_unref (folder->job);
+      folder->job = NULL;
     }
 
   /* disconnect from the corresponding file */
@@ -285,6 +277,10 @@ thunar_folder_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_CORRESPONDING_FILE:
+      g_value_set_object (value, folder->corresponding_file);
+      break;
+
     case PROP_LOADING:
       g_value_set_boolean (value, thunar_folder_get_loading (folder));
       break;
@@ -298,13 +294,38 @@ thunar_folder_get_property (GObject    *object,
 
 
 static void
-thunar_folder_error (ThunarVfsJob *job,
+thunar_folder_set_property (GObject      *object,
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
+{
+  ThunarFolder *folder = THUNAR_FOLDER (object);
+
+  switch (prop_id)
+    {
+    case PROP_CORRESPONDING_FILE:
+      folder->corresponding_file = g_value_dup_object (value);
+      break;
+
+    case PROP_LOADING:
+      _thunar_assert_not_reached ();
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
+
+static void
+thunar_folder_error (ExoJob       *job,
                      GError       *error,
                      ThunarFolder *folder)
 {
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
-  _thunar_return_if_fail (THUNAR_VFS_IS_JOB (job));
-  _thunar_return_if_fail (folder->job == job);
+  _thunar_return_if_fail (THUNAR_IS_JOB (job));
 
   /* tell the consumer about the problem */
   g_signal_emit (G_OBJECT (folder), folder_signals[ERROR], 0, error);
@@ -313,42 +334,25 @@ thunar_folder_error (ThunarVfsJob *job,
 
 
 static gboolean
-thunar_folder_infos_ready (ThunarVfsJob *job,
-                           GList        *infos,
+thunar_folder_files_ready (ThunarJob    *job,
+                           GList        *files,
                            ThunarFolder *folder)
 {
-  ThunarFile *file;
-  GList      *lp;
-
   _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), FALSE);
-  _thunar_return_val_if_fail (THUNAR_VFS_IS_JOB (job), FALSE);
-  _thunar_return_val_if_fail (folder->handle == NULL, FALSE);
-  _thunar_return_val_if_fail (folder->job == job, FALSE);
-
-  /* turn the info list into a file list */
-  for (lp = infos; lp != NULL; lp = lp->next)
-    {
-      /* get the file corresponding to the info... */
-      file = thunar_file_get_for_info (lp->data);
-
-      /* ...release the info at the list position... */
-      thunar_vfs_info_unref (lp->data);
-
-      /* ...and replace it with the file */
-      lp->data = file;
-    }
+  _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
+  _thunar_return_val_if_fail (folder->monitor == NULL, FALSE);
 
   /* merge the list with the existing list of new files */
-  folder->new_files = g_list_concat (folder->new_files, infos);
+  folder->new_files = g_list_concat (folder->new_files, files);
 
-  /* TRUE to indicate that we took over ownership of the infos list */
+  /* indicate that we took over ownership of the file list */
   return TRUE;
 }
 
 
 
 static void
-thunar_folder_finished (ThunarVfsJob *job,
+thunar_folder_finished (ExoJob       *job,
                         ThunarFolder *folder)
 {
   ThunarFile *file;
@@ -356,10 +360,9 @@ thunar_folder_finished (ThunarVfsJob *job,
   GList      *lp;
 
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
-  _thunar_return_if_fail (THUNAR_VFS_IS_JOB (job));
+  _thunar_return_if_fail (THUNAR_IS_JOB (job));
   _thunar_return_if_fail (THUNAR_IS_FILE (folder->corresponding_file));
-  _thunar_return_if_fail (folder->handle == NULL);
-  _thunar_return_if_fail (folder->job == job);
+  _thunar_return_if_fail (folder->monitor == NULL);
 
   /* check if we need to merge new files with existing files */
   if (G_UNLIKELY (folder->files != NULL))
@@ -426,18 +429,23 @@ thunar_folder_finished (ThunarVfsJob *job,
       folder->files = folder->new_files;
       folder->new_files = NULL;
 
-      /* emit a "files-added" signal for the new files */
-      g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, folder->files);
+      if (folder->files != NULL)
+        {
+          /* emit a "files-added" signal for the new files */
+          g_signal_emit (G_OBJECT (folder), folder_signals[FILES_ADDED], 0, folder->files);
+        }
     }
 
   /* we did it, the folder is loaded */
   g_signal_handlers_disconnect_matched (folder->job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
-  g_object_unref (G_OBJECT (folder->job));
+  g_object_unref (folder->job);
   folder->job = NULL;
 
   /* add us to the file alteration monitor */
-  folder->handle = thunar_vfs_monitor_add_directory (folder->monitor, thunar_file_get_path (folder->corresponding_file),
-                                                     thunar_folder_monitor, folder);
+  folder->monitor = g_file_monitor_directory (thunar_file_get_file (folder->corresponding_file), 
+                                              G_FILE_MONITOR_NONE, NULL, NULL);
+  if (G_LIKELY (folder->monitor != NULL))
+    g_signal_connect (folder->monitor, "changed", G_CALLBACK (thunar_folder_monitor), folder);
 
   /* tell the consumers that we have loaded the directory */
   g_object_notify (G_OBJECT (folder), "loading");
@@ -504,37 +512,35 @@ thunar_folder_file_destroyed (ThunarFileMonitor *file_monitor,
 
 
 static void
-thunar_folder_monitor (ThunarVfsMonitor       *monitor,
-                       ThunarVfsMonitorHandle *handle,
-                       ThunarVfsMonitorEvent   event,
-                       ThunarVfsPath          *handle_path,
-                       ThunarVfsPath          *event_path,
-                       gpointer                user_data)
+thunar_folder_monitor (GFileMonitor     *monitor,
+                       GFile            *event_file,
+                       GFile            *other_file,
+                       GFileMonitorEvent event_type,
+                       gpointer          user_data)
 {
   ThunarFolder *folder = THUNAR_FOLDER (user_data);
   ThunarFile   *file;
   GList        *lp;
   GList         list;
 
-  _thunar_return_if_fail (THUNAR_VFS_IS_MONITOR (monitor));
+  _thunar_return_if_fail (G_IS_FILE_MONITOR (monitor));
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
   _thunar_return_if_fail (folder->monitor == monitor);
-  _thunar_return_if_fail (folder->handle == handle);
   _thunar_return_if_fail (folder->job == NULL);
 
   /* check on which file the event occurred */
-  if (!thunar_vfs_path_equal (event_path, thunar_file_get_path (folder->corresponding_file)))
+  if (!g_file_equal (event_file, thunar_file_get_file (folder->corresponding_file)))
     {
       /* check if we already ship the file */
       for (lp = folder->files; lp != NULL; lp = lp->next)
-        if (thunar_vfs_path_equal (event_path, thunar_file_get_path (lp->data)))
+        if (g_file_equal (event_file, thunar_file_get_file (lp->data)))
           break;
 
       /* if we don't have it, add it if the event is not an "deleted" event */
-      if (G_UNLIKELY (lp == NULL && event != THUNAR_VFS_MONITOR_EVENT_DELETED))
+      if (G_UNLIKELY (lp == NULL && event_type != G_FILE_MONITOR_EVENT_DELETED))
         {
           /* allocate a file for the path */
-          file = thunar_file_get_for_path (event_path, NULL);
+          file = thunar_file_get (event_file, NULL);
           if (G_UNLIKELY (file == NULL))
             return;
 
@@ -548,7 +554,7 @@ thunar_folder_monitor (ThunarVfsMonitor       *monitor,
       else if (lp != NULL)
         {
           /* update/destroy the file */
-          if (event == THUNAR_VFS_MONITOR_EVENT_DELETED)
+          if (event_type == G_FILE_MONITOR_EVENT_DELETED)
             thunar_file_destroy (lp->data);
           else
             thunar_file_reload (lp->data);
@@ -557,7 +563,7 @@ thunar_folder_monitor (ThunarVfsMonitor       *monitor,
   else
     {
       /* update/destroy the corresponding file */
-      if (event == THUNAR_VFS_MONITOR_EVENT_DELETED)
+      if (event_type == G_FILE_MONITOR_EVENT_DELETED)
         thunar_file_destroy (folder->corresponding_file);
       else
         thunar_file_reload (folder->corresponding_file);
@@ -601,12 +607,10 @@ thunar_folder_get_for_file (ThunarFile *file)
   else
     {
       /* allocate the new instance */
-      folder = g_object_new (THUNAR_TYPE_FOLDER, NULL);
-      folder->corresponding_file = file;
-      g_object_ref (G_OBJECT (file));
+      folder = g_object_new (THUNAR_TYPE_FOLDER, "corresponding-file", file, NULL);
 
       /* drop the floating reference */
-      exo_gtk_object_ref_sink (GTK_OBJECT (folder));
+      g_object_ref_sink (G_OBJECT (folder));
 
       /* connect the folder to the file */
       g_object_set_qdata (G_OBJECT (file), thunar_folder_quark, folder);
@@ -694,15 +698,16 @@ thunar_folder_reload (ThunarFolder *folder)
     {
       /* disconnect from the job */
       g_signal_handlers_disconnect_matched (folder->job, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, folder);
-      thunar_vfs_job_cancel (THUNAR_VFS_JOB (folder->job));
-      g_object_unref (G_OBJECT (folder->job));
+      g_object_unref (folder->job);
+      folder->job = NULL;
     }
 
   /* disconnect from the file alteration monitor */
-  if (G_UNLIKELY (folder->handle != NULL))
+  if (G_UNLIKELY (folder->monitor != NULL))
     {
-      thunar_vfs_monitor_remove (folder->monitor, folder->handle);
-      folder->handle = NULL;
+      g_file_monitor_cancel (folder->monitor);
+      g_object_unref (folder->monitor);
+      folder->monitor = NULL;
     }
 
   /* reset the new_files list */
@@ -710,10 +715,10 @@ thunar_folder_reload (ThunarFolder *folder)
   folder->new_files = NULL;
 
   /* start a new job */
-  folder->job = thunar_vfs_listdir (thunar_file_get_path (folder->corresponding_file), NULL);
+  folder->job = thunar_io_jobs_list_directory (thunar_file_get_file (folder->corresponding_file));
   g_signal_connect (folder->job, "error", G_CALLBACK (thunar_folder_error), folder);
   g_signal_connect (folder->job, "finished", G_CALLBACK (thunar_folder_finished), folder);
-  g_signal_connect (folder->job, "infos-ready", G_CALLBACK (thunar_folder_infos_ready), folder);
+  g_signal_connect (folder->job, "files-ready", G_CALLBACK (thunar_folder_files_ready), folder);
 
   /* tell all consumers that we're loading */
   g_object_notify (G_OBJECT (folder), "loading");
